/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://gre/modules/utils.js");

// define the bookmark tree
var Book = {
	
	historyService : null,
	bookmarkService : null,
	livemarkService : null,

	RESULT_TYPE_URI			: 0,
	RESULT_TYPE_FOLDER		: 6,
	RESULT_TYPE_SEPARATOR	: 7,

	TOOLBAR		: null,
	MENUFOLDER	: null,

	BASE_TYPE_ARRAY		: 0,
	BASE_TYPE_HASH		: 1,
	BASE_TYPE_IDHASH	: 2,
	BASE_TYPE_TREE		: 3,

	TYPE_BOOKMARK		: 0,
	TYPE_FOLDER			: 1,
	TYPE_LIVEMARK		: 2,
	TYPE_SEPARATOR		: 3,
	TYPE_UNKNOWN 		: 4,

	init : function(){
		this.historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
							.getService(Components.interfaces.nsINavHistoryService);

		this.bookmarkService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
							.getService(Components.interfaces.nsINavBookmarksService);

		this.livemarkService = Components.classes["@mozilla.org/browser/livemark-service;2"]
							.getService(Components.interfaces.nsILivemarkService);
							
		this.TOOLBAR = this.bookmarkService.toolbarFolder;
		this.MENUFOLDER	= this.bookmarkService.bookmarksMenuFolder;
	},
	
	_init : function(){
		this.historyService = null;
		this.bookmarkService = null;
		this.livemarkService = null;
		this.TOOLBAR = null;
		this.MENUFOLDER = null;
	},
	
	//+------------------------------------------------------------------------------------
	// Base types
	//+------------------------------------------------------------------------------------

	getBookmarkOptions : function(){
		var options = this.historyService.getNewQueryOptions();
		options.queryType = options.QUERY_TYPE_BOOKMARKS; 
		options.includeHidden = true;
		return options;
	},

	getBookmarkQuery : function(folders){
		var query = this.historyService.getNewQuery();
		query.setFolders(folders, folders.length);
		
		return query;
	},

	isBookmarked : function(url){
		return PlacesUtils.bookmarks.isBookmarked(getURLAsURI(url));
	},
	
	getParent : function(url){
		var folders = [this.MENUFOLDER];
		
		var query = this.getBookmarkQuery(folders);
		var options = this.getBookmarkOptions();
		options.expandQueries = true;
		
		var result = this.historyService.executeQuery(query, options);
		var node = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		
		return this.findParent(node, url);
	},
	
	findParent : function(node, uri){
		var parent = null;
		if(PlacesUtils.nodeIsFolder(node)){
			node = node.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
			
			node.containerOpen = true;
			var size = node.childCount;
			for(var i = 0 ;i < size; i++){
				var found = this.findParent(node.getChild(i), uri);
				if(found != null) parent = found;
			}
			node.containerOpen = false;
		}else {
			if(node.uri == uri)
				parent = node.parent;
		}
		return parent;
	},
	
	update : function(hash, tree){
		var folders = [this.TOOLBAR, this.MENUFOLDER];
		var queries = [];
		for(var i in hash){
			var query = this.historyService.getNewQuery();
			query.uri = getURLAsURI(i);
			query.setFolders(folders, folders.length);
			
			queries.push(query);
		}
	
		var options = this.getBookmarkOptions();

		var placeURIs = PlacesUtils.history.queriesToQueryString(queries, queries.length, options);
	
		tree.place = placeURIs;
	},
	
	test : function(list){
		var node;
		var folders = [this.TOOLBAR, this.MENUFOLDER];
		var queries = [];
		while((node = list.shift())){
			var query = this.historyService.getNewQuery();
			query.uri = getURLAsURI(node.uri);
			query.setFolders(folders, folders.length);
			queries.push(query);
		}
		var options = this.getBookmarkOptions();
		var placeURIs = PlacesUtils.history.queriesToQueryString(queries, queries.length, options);
		
		document.getElementById("placesList").place = placeURIs;
	},
	
	getMostVisited : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery([6]);
		var result = this.historyService.executeQuery(query, options);

  		return this.allToBaseType(this.BASE_TYPE_ARRAY, new Array(), result);
	},
	
	getLastVisited : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery([7]);
		var result = this.historyService.executeQuery(query, options);

  		return this.allToBaseType(this.BASE_TYPE_ARRAY, new Array(), result);
	},
	
	getTags : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery([8]);
		var result = this.historyService.executeQuery(query, options);

  		return this.allToBaseType(this.BASE_TYPE_ARRAY, new Array(), result);
	},
	
	getAllResourcesList : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery([this.TOOLBAR, this.MENUFOLDER]);
		var result = this.historyService.executeQuery(query, options);

		return this.toBaseType(this.BASE_TYPE_ARRAY, new Array(), result);
	},
	
	getAllResourcesHash : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery([this.TOOLBAR, this.MENUFOLDER]);
		var result = this.historyService.executeQuery(query, options);

		return this.toBaseType(this.BASE_TYPE_HASH, new Array(), result);
	},

	getIDHash : function(){
		var options = this.getBookmarkOptions();

		var query = this.getBookmarkQuery(this.TOOLBAR);
		var result = this.historyService.executeQuery(query, options);

		return this.allToBaseType(this.BASE_TYPE_IDHASH, new Array(), result);
	},

	allToBaseType : function(type, array, result){
		var root = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		root.containerOpen = true;

		var size = root.childCount;
		for (var i = 0; i < size; i++) {		
			this.collectAllNodes(type, root.getChild(i), array);
		}

		root.containerOpen = false;
  		return array;
	},

	toBaseType : function(type, array, result){
		var root = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		root.containerOpen = true;

		var size = root.childCount;
		for (var i = 0; i < size; i++) {		
			this.collectNodes(type, root.getChild(i), array);
		}

		root.containerOpen = false;
  		return array;
	},

	collectNodes : function(type, node, array){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				this.put(type, node, array);
				break;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					this.put(type, node, array);
				}else {
					node.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);

					node.containerOpen = true;

					var size = node.childCount;
					for (var i = 0; i < size; i++) {
						this.collectNodes(type, node.getChild(i), array);
					}

					node.containerOpen = false;
				}
				break;
			}
			default : {
				break;
			}
		}
	},

	collectAllNodes : function(type, node, array){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				this.put(type, node, array);
				break;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					this.put(type, node, array);
				}else {
					node.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
					
					node.containerOpen = true;

					this.put(type, node, array);

					var size = node.childCount;
					for (var i = 0; i < size; i++) {
						this.collectNodes(type, node.getChild(i), array);
					}

					node.containerOpen = false;
				}
				break;
			}
			default : {
				break;
			}
		}
	},

	put : function(type, node, array){
		switch(type){
			case this.BASE_TYPE_ARRAY : {
				array.push(node);
				break;
			}
			case this.BASE_TYPE_HASH : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					array[this.resolveURL(livemarkService.getFeedURI(node.itemId))] = node;
				}else {
					array[node.uri] = node;
				}
				break;
			}
			case this.BASE_TYPE_IDHASH : {
				array[node.itemId] = node;
				break;
			}
			case this.BASE_TYPE_TREE : {
				//TODO
				break;
			}
			default : {
				alert("Unknown base type : " + type);
				break;
			}
		}
	},

	resolveURL : function(nsIUri){
		var host = nsIUri.host;
		var path = nsIUri.path;
		var protocol = "http://";
		return protocol + host + "/" + path;
	},

	//+------------------------------------------------------------------------------------
	// Add
	//+------------------------------------------------------------------------------------

	addBookmark : function(name, url){
		var folderID = this.bookmarkService.bookmarksMenuFolder;

		var ios = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI(url, null, null);

		this.bookmarkService.insertBookmark(folderID, uri, -1, name);
	},

	//TODO
	addFolder : function(){

	},

	//TODO
	addLivemark : function(){

	},

	//TODO
	addSeparator : function(){

	},

	//+------------------------------------------------------------------------------------
	// Remove
	//+------------------------------------------------------------------------------------

	//TODO: test
	removeBookmark : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeFolder : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeLivemark : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeSeparator : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//+------------------------------------------------------------------------------------
	// Properties
	//+------------------------------------------------------------------------------------

	//TODO: test
	getType : function(node){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				return this.TYPE_BOOKMARK;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					return this.TYPE_LIVEMARK;
				}
				return this.TYPE_FOLDER;
			}
			case this.RESULT_TYPE_SEPARATOR : {
				return this.TYPE_SEPARATOR;
			}
			default : {
				return this.TYPE_UNKNOWN;
			}
		}
	},

	getName : function(node){
		var title = node.title;
		return title ? title : "";
	},

	getUrl : function(node){
		var uri = node.uri;
		return uri ? uri : "";
	},

	getFUrl : function(node){
		var furl = this.resolveURL(this.livemarkService.getFeedURI(node.itemId));
		return furl ? furl : "";
	},

	getURI : function(node){
		if(this.isLivemark(node)){
			return this.getFUrl(node);
		}
		return this.getUrl(node);
	},

	getKeys : function(node){
		var keys = PlacesUtils.bookmarks.getKeywordForBookmark(node.itemId);
		return keys ? keys : "";
	},

	getAddDate : function(node){
		var ad = node.dateAdded;
		return ad ? ad : "";
	},

	getLMDate : function(node){
		var lm = node.lastModified;
		return lm ? lm : -1;
	},

	//TODO
	getLVDate : function(node){

		return "";
	},

	getDesc : function(node){
		var desc = PlacesUIUtils.getItemDescription(node.itemId);
		return desc ? desc : "";
	},

	//TODO: add default icon
	getIcon : function(node){
		var icon = node.icon;
		return icon ? icon.spec : "";
	},
	
	isLivemarkItem : function(node){
		var parent = node.parent;
		parent.QueryInterface(Components.interfaces.nsINavHistoryResultNode);

		return PlacesUtils.nodeIsLivemarkContainer(parent);
	},
	
	isLivemark : function(node){
		node.QueryInterface(Components.interfaces.nsINavHistoryResultNode);
		
		return PlacesUtils.nodeIsLivemarkContainer(node);
	},
	
	isMenuFolder : function(node){
		return node.itemId == this.MENUFOLDER;
	},
	
	isToolbarFolder : function(node){
		return node.itemId == this.TOOLBAR;
	}

};

Book.init();
